import { IColumnDesc } from '../model';
import Column from '../model/Column';
import ADataProvider from '../provider/ADataProvider';
import LocalDataProvider from '../provider/LocalDataProvider';
import LineUp from '../ui/LineUp';
import Taggle from '../ui/taggle/Taggle';
import ColumnBuilder from './column/ColumnBuilder';
import LineUpBuilder from './LineUpBuilder';
import RankingBuilder from './RankingBuilder';
/**
 * builder for a LocalDataProvider along with LineUp configuration options
 */
export default class DataBuilder extends LineUpBuilder {
    private readonly data;
    private readonly columns;
    private readonly providerOptions;
    private readonly rankBuilders;
    private _deriveColors;
    constructor(data: object[]);
    /**
     * allow just a single selection
     */
    singleSelection(): this;
    /**
     * filter all rankings by all filters in LineUp
     */
    filterGlobally(): this;
    /**
     * no limits for the number of sorting and grouping criteria
     */
    noCriteriaLimits(): this;
    /**
     * limit the number of sort and grouping criterias i.e the nesting level
     * @param {number} sortingCritera
     * @param {number} groupingCriteria
     * @returns {this}
     */
    limitCriteria(sortingCritera: number, groupingCriteria: number): this;
    /**
     * triggers to derive the column descriptions for the given data
     * @param {string} columns optional enforced order of columns
     */
    deriveColumns(...columns: string[]): this;
    /**
     * tirggers to assign colors for the given descriptions
     */
    deriveColors(): this;
    /**
     * register another column type to this data provider
     * @param {string} type unique type id
     * @param {typeof Column} clazz column class
     */
    registerColumnType(type: string, clazz: typeof Column): this;
    /**
     * push another column description to this data provider
     * @param {IColumnDesc | ColumnBuilder} column column description or builder instance
     */
    column(column: IColumnDesc | ColumnBuilder): this;
    /**
     * restores a given ranking dump
     * @param dump dump as created using '.dump()'
     */
    restore(dump: any): this;
    /**
     * add the default ranking (all columns) to this data provider
     * @param {boolean} addSupportTypes add support types, too, default: true
     */
    defaultRanking(addSupportTypes?: boolean): this;
    /**
     * add another ranking to this data provider
     * @param {((data: ADataProvider) => void) | RankingBuilder} builder ranking builder
     */
    ranking(builder: ((data: ADataProvider) => void) | RankingBuilder): this;
    /**
     * builds the data provider itself
     * @returns {LocalDataProvider}
     */
    buildData(): LocalDataProvider;
    /**
     * builds LineUp at the given parent DOM node
     * @param {HTMLElement} node parent DOM node to attach
     * @returns {LineUp}
     */
    build(node: HTMLElement): LineUp;
    /**
     * builds Taggle at the given parent DOM node
     * @param {HTMLElement} node parent DOM node to attach
     * @returns {Taggle}
     */
    buildTaggle(node: HTMLElement): Taggle;
}
/**
 * creates a new builder instance for the given data
 * @param {object[]} arr data to visualize
 * @returns {DataBuilder}
 */
export declare function builder(arr: object[]): DataBuilder;
