/**
 * main module of LineUp.js containing the main class and exposes all other modules
 */
import './style.scss';
import { IColumnDesc } from './model';
import { ILocalDataProviderOptions } from './provider';
import ADataProvider from './provider/ADataProvider';
import LocalDataProvider from './provider/LocalDataProvider';
import LineUp, { ILineUpOptions } from './ui/LineUp';
import Taggle, { ITaggleOptions } from './ui/taggle';
export * from './provider';
export * from './renderer';
export * from './model';
export * from './builder';
export * from './ui';
export * from './interfaces';
export * from './config';
export { default } from './ui/LineUp';
export { IBoxPlotData, IAdvancedBoxPlotData, IStatistics, ICategoricalBin, ICategoricalStatistics, INumberBin } from './internal/math';
/**
 * LineUp version
 * @type {string}
 */
export declare const version: string;
/**
 * LineUp unique build id
 * @type {string}
 */
export declare const buildId: string;
/**
 * LineUp license type
 * @type {string}
 */
export declare const license: string;
export declare function createLocalDataProvider(data: any[], columns: IColumnDesc[], options?: Partial<ILocalDataProviderOptions>): LocalDataProvider;
/**
 *
 * @param container the html element lineup should be built in
 * @param data {ADataProvider} the data providier
 * @param config {Partial<ILineUpOptions>} lineup configuration overrides
 * @returns {LineUp} the created lineup instance
 */
export declare function createLineUp(container: HTMLElement, data: ADataProvider, config?: Partial<ILineUpOptions>): LineUp;
export declare function createTaggle(container: HTMLElement, data: ADataProvider, config?: Partial<ITaggleOptions>): Taggle;
