import { IAdvancedBoxPlotData, IBoxPlotData } from '../internal';
import Column from './Column';
import { IArrayColumn } from './IArrayColumn';
import { IColumnDesc, IDataRow } from './interfaces';
import { IMapAbleColumn, IMappingFunction } from './MappingFunction';
export interface INumberColumn extends Column {
    getNumber(row: IDataRow): number;
    getRawNumber(row: IDataRow): number;
}
export default INumberColumn;
export interface INumberDesc {
    /**
     * dump of mapping function
     */
    map?: any;
    /**
     * either map or domain should be available
     */
    domain?: [number, number];
    /**
     * @default [0,1]
     */
    range?: [number, number];
    /**
     * d3 formatting option
     * @default .3n
     */
    numberFormat?: string;
    /**
     * missing value to use
     * @default 0
     */
    missingValue?: number;
}
/**
 * checks whether the given column or description is a number column, i.e. the value is a number
 * @param col
 * @returns {boolean}
 */
export declare function isNumberColumn(col: Column): col is INumberColumn;
export declare function isNumberColumn(col: IColumnDesc): col is INumberDesc & IColumnDesc;
export declare function getBoxPlotNumber(col: IBoxPlotColumn, row: IDataRow, mode: 'raw' | 'normalized'): number;
export declare enum ESortMethod {
    min = "min",
    max = "max",
    median = "median",
    q1 = "q1",
    q3 = "q3"
}
export interface IBoxPlotColumn extends INumberColumn, IMapAbleColumn {
    getBoxPlotData(row: IDataRow): IBoxPlotData | null;
    getMapping(): IMappingFunction;
    getRawBoxPlotData(row: IDataRow): IBoxPlotData | null;
    getSortMethod(): string;
    setSortMethod(sortMethod: string): void;
}
export declare function isBoxPlotColumn(col: Column): col is IBoxPlotColumn;
export declare enum EAdvancedSortMethod {
    min = "min",
    max = "max",
    median = "median",
    q1 = "q1",
    q3 = "q3",
    mean = "mean"
}
export interface IAdvancedBoxPlotColumn extends IBoxPlotColumn {
    getBoxPlotData(row: IDataRow): IAdvancedBoxPlotData | null;
    getRawBoxPlotData(row: IDataRow): IAdvancedBoxPlotData | null;
}
export interface INumbersColumn extends IAdvancedBoxPlotColumn, IArrayColumn<number> {
    getNumbers(row: IDataRow): number[];
    getRawNumbers(row: IDataRow): number[];
}
export declare function isNumbersColumn(col: Column): col is INumbersColumn;
export interface INumberFilter {
    min: number;
    max: number;
    filterMissing: boolean;
}
