import Column, { widthChanged, labelChanged, metaDataChanged, dirty, dirtyHeader, dirtyValues, rendererTypeChanged, groupRendererChanged, summaryRendererChanged, visibilityChanged } from './Column';
import { ICategoricalColumn, ICategoricalDesc, ICategoricalFilter, ICategory } from './ICategoricalColumn';
import { IDataRow } from './interfaces';
import { INumberColumn } from './NumberColumn';
import ValueColumn, { IValueColumnDesc, dataLoaded } from './ValueColumn';
export declare type ICategoricalNumberColumnDesc = ICategoricalDesc & IValueColumnDesc<number>;
/**
 * emitted when the mapping property changes
 * @asMemberOf OrdinalColumn
 * @event
 */
export declare function mappingChanged(previous: number[], current: number[]): void;
/**
 * emitted when the filter property changes
 * @asMemberOf OrdinalColumn
 * @event
 */
export declare function filterChanged(previous: ICategoricalFilter | null, current: ICategoricalFilter | null): void;
/**
 * similar to a categorical column but the categories are mapped to numbers
 */
export default class OrdinalColumn extends ValueColumn<number> implements INumberColumn, ICategoricalColumn {
    static readonly EVENT_MAPPING_CHANGED = "mappingChanged";
    static readonly EVENT_FILTER_CHANGED = "filterChanged";
    readonly categories: ICategory[];
    private missingCategory;
    private readonly lookup;
    private currentFilter;
    constructor(id: string, desc: Readonly<ICategoricalNumberColumnDesc>);
    protected createEventList(): string[];
    on(type: typeof OrdinalColumn.EVENT_MAPPING_CHANGED, listener: typeof mappingChanged | null): this;
    on(type: typeof OrdinalColumn.EVENT_FILTER_CHANGED, listener: typeof filterChanged | null): this;
    on(type: typeof ValueColumn.EVENT_DATA_LOADED, listener: typeof dataLoaded | null): this;
    on(type: typeof Column.EVENT_WIDTH_CHANGED, listener: typeof widthChanged | null): this;
    on(type: typeof Column.EVENT_LABEL_CHANGED, listener: typeof labelChanged | null): this;
    on(type: typeof Column.EVENT_METADATA_CHANGED, listener: typeof metaDataChanged | null): this;
    on(type: typeof Column.EVENT_DIRTY, listener: typeof dirty | null): this;
    on(type: typeof Column.EVENT_DIRTY_HEADER, listener: typeof dirtyHeader | null): this;
    on(type: typeof Column.EVENT_DIRTY_VALUES, listener: typeof dirtyValues | null): this;
    on(type: typeof Column.EVENT_RENDERER_TYPE_CHANGED, listener: typeof rendererTypeChanged | null): this;
    on(type: typeof Column.EVENT_GROUP_RENDERER_TYPE_CHANGED, listener: typeof groupRendererChanged | null): this;
    on(type: typeof Column.EVENT_SUMMARY_RENDERER_TYPE_CHANGED, listener: typeof summaryRendererChanged | null): this;
    on(type: typeof Column.EVENT_VISIBILITY_CHANGED, listener: typeof visibilityChanged | null): this;
    readonly dataLength: number;
    readonly labels: string[];
    getValue(row: IDataRow): number;
    getCategory(row: IDataRow): ICategory | null;
    getColor(row: IDataRow): any;
    getLabel(row: IDataRow): any;
    getLabels(row: IDataRow): any;
    getValues(row: IDataRow): any;
    getMap(row: IDataRow): any;
    getMapLabel(row: IDataRow): any;
    getSet(row: IDataRow): any;
    getNumber(row: IDataRow): number;
    isMissing(row: IDataRow): any;
    getRawNumber(row: IDataRow): number;
    getExportValue(row: IDataRow, format: 'text' | 'json'): any;
    dump(toDescRef: (desc: any) => any): any;
    restore(dump: any, factory: (dump: any) => Column | null): void;
    getMapping(): number[];
    setMapping(mapping: number[]): void;
    isFiltered(): boolean;
    filter(row: IDataRow): boolean;
    group(row: IDataRow): any;
    getFilter(): ICategoricalFilter | null;
    setFilter(filter: ICategoricalFilter | null): any;
    compare(a: IDataRow, b: IDataRow): any;
    getRenderer(): string;
}
