import Column, { labelChanged, metaDataChanged, dirty, widthChanged, dirtyHeader, dirtyValues, rendererTypeChanged, groupRendererChanged, summaryRendererChanged, visibilityChanged } from './Column';
import { IArrayColumn } from './IArrayColumn';
import { ICategoricalDesc, ICategoricalFilter, ICategory } from './ICategoricalColumn';
import { IDataRow } from './interfaces';
import ValueColumn, { IValueColumnDesc, dataLoaded } from './ValueColumn';
export interface ISetDesc extends ICategoricalDesc {
    separator?: string;
}
export declare type ISetColumnDesc = ISetDesc & IValueColumnDesc<string[]>;
/**
 * emitted when the filter property changes
 * @asMemberOf SetColumn
 * @event
 */
export declare function filterChanged(previous: ICategoricalFilter | null, current: ICategoricalFilter | null): void;
/**
 * a string column with optional alignment
 */
export default class SetColumn extends ValueColumn<string[]> implements IArrayColumn<boolean> {
    static readonly EVENT_FILTER_CHANGED = "filterChanged";
    readonly categories: ICategory[];
    private readonly separator;
    private readonly lookup;
    /**
     * set of categories to show
     * @type {null}
     * @private
     */
    private currentFilter;
    constructor(id: string, desc: Readonly<ISetColumnDesc>);
    protected createEventList(): string[];
    on(type: typeof SetColumn.EVENT_FILTER_CHANGED, listener: typeof filterChanged | null): this;
    on(type: typeof ValueColumn.EVENT_DATA_LOADED, listener: typeof dataLoaded | null): this;
    on(type: typeof Column.EVENT_WIDTH_CHANGED, listener: typeof widthChanged | null): this;
    on(type: typeof Column.EVENT_LABEL_CHANGED, listener: typeof labelChanged | null): this;
    on(type: typeof Column.EVENT_METADATA_CHANGED, listener: typeof metaDataChanged | null): this;
    on(type: typeof Column.EVENT_DIRTY, listener: typeof dirty | null): this;
    on(type: typeof Column.EVENT_DIRTY_HEADER, listener: typeof dirtyHeader | null): this;
    on(type: typeof Column.EVENT_DIRTY_VALUES, listener: typeof dirtyValues | null): this;
    on(type: typeof Column.EVENT_RENDERER_TYPE_CHANGED, listener: typeof rendererTypeChanged | null): this;
    on(type: typeof Column.EVENT_GROUP_RENDERER_TYPE_CHANGED, listener: typeof groupRendererChanged | null): this;
    on(type: typeof Column.EVENT_SUMMARY_RENDERER_TYPE_CHANGED, listener: typeof summaryRendererChanged | null): this;
    on(type: typeof Column.EVENT_VISIBILITY_CHANGED, listener: typeof visibilityChanged | null): this;
    readonly labels: string[];
    readonly dataLength: number;
    getValue(row: IDataRow): string[];
    getLabel(row: IDataRow): string;
    private normalize;
    getSet(row: IDataRow): Set<ICategory>;
    getCategories(row: IDataRow): ICategory[];
    isMissing(row: IDataRow): boolean;
    getValues(row: IDataRow): boolean[];
    getLabels(row: IDataRow): string[];
    getMap(row: IDataRow): {
        key: string;
        value: boolean;
    }[];
    getMapLabel(row: IDataRow): {
        key: string;
        value: string;
    }[];
    dump(toDescRef: (desc: any) => any): any;
    restore(dump: any, factory: (dump: any) => Column | null): void;
    isFiltered(): boolean;
    filter(row: IDataRow): boolean;
    getFilter(): any;
    setFilter(filter: ICategoricalFilter | null): any;
    compare(a: IDataRow, b: IDataRow): number;
}
