import { IColumnDesc, IDataRow } from '../model';
import { IOrderedGroup } from '../model/Group';
import Ranking from '../model/Ranking';
import ADataProvider, { IDataProviderOptions } from './ADataProvider';
/**
 * common base implementation of a DataProvider with a fixed list of column descriptions
 */
declare abstract class ACommonDataProvider extends ADataProvider {
    private columns;
    private rankingIndex;
    constructor(columns?: IColumnDesc[], options?: Partial<IDataProviderOptions>);
    protected rankAccessor(row: IDataRow, _id: string, _desc: IColumnDesc, ranking: Ranking): number;
    /**
     * returns the maximal number of nested/hierarchical sorting criteria
     * @return {number}
     */
    protected getMaxNestedSortingCriteria(): number;
    protected getMaxGroupColumns(): number;
    cloneRanking(existing?: Ranking): Ranking;
    sort(ranking: Ranking): Promise<IOrderedGroup[]> | IOrderedGroup[];
    protected abstract sortImpl(ranking: Ranking): Promise<IOrderedGroup[]> | IOrderedGroup[];
    /**
     * adds another column description to this data provider
     * @param column
     */
    pushDesc(column: IColumnDesc): void;
    clearColumns(): void;
    getColumns(): IColumnDesc[];
    findDesc(ref: string): IColumnDesc;
    /**
     * identify by the tuple type@columnname
     * @param desc
     * @returns {string}
     */
    toDescRef(desc: any): any;
    /**
     * generates a default ranking by using all column descriptions ones
     */
    deriveDefault(addSupportType?: boolean): Ranking;
    fromDescRef(descRef: any): any;
    restore(dump: any): void;
    nextRankingId(): string;
}
export default ACommonDataProvider;
