import Column, { IColumnDesc, IDataRow, INumberColumn, IOrderedGroup } from '../model';
import Ranking from '../model/Ranking';
import ACommonDataProvider from './ACommonDataProvider';
import { IDataProviderOptions, IStatsBuilder } from './ADataProvider';
export interface ILocalDataProviderOptions {
    /**
     * whether the filter should be applied to all rankings regardless where they are
     * default: false
     */
    filterGlobally: boolean;
    /**
     * jump to search results such that they are visible
     * default: false
     */
    jumpToSearchResult: boolean;
    /**
     * the maximum number of nested sorting criteria
     */
    maxNestedSortingCriteria: number;
    maxGroupColumns: number;
}
/**
 * a data provider based on an local array
 */
export default class LocalDataProvider extends ACommonDataProvider {
    private _data;
    private options;
    private readonly reorderAll;
    private _dataRows;
    private filter;
    constructor(_data: any[], columns?: IColumnDesc[], options?: Partial<ILocalDataProviderOptions & IDataProviderOptions>);
    /**
     * set a globally applied filter to all data without changing the data source itself
     * @param {((row: IDataRow) => boolean) | null} filter
     */
    setFilter(filter: ((row: IDataRow) => boolean) | null): void;
    getFilter(): ((row: IDataRow) => boolean) | null;
    getTotalNumberOfRows(): number;
    protected getMaxGroupColumns(): number;
    protected getMaxNestedSortingCriteria(): number;
    readonly data: any[];
    /**
     * replaces the dataset rows with a new one
     * @param data
     */
    setData(data: any[]): void;
    clearData(): void;
    /**
     * append rows to the dataset
     * @param data
     */
    appendData(data: any[]): void;
    cloneRanking(existing?: Ranking): Ranking;
    cleanUpRanking(ranking: Ranking): void;
    sortImpl(ranking: Ranking): IOrderedGroup[];
    viewRaw(indices: number[]): any[];
    viewRawRows(indices: number[]): IDataRow[];
    view(indices: number[]): any[];
    fetch(orders: number[][]): IDataRow[][];
    /**
     * helper for computing statistics
     * @param indices
     * @returns {{stats: (function(INumberColumn): *), hist: (function(ICategoricalColumn): *)}}
     */
    stats(indices?: number[]): IStatsBuilder;
    mappingSample(col: INumberColumn): number[];
    searchAndJump(search: string | RegExp, col: Column): void;
}
