import Column, { IColumnDesc, IDataRow } from '../model';
import { IOrderedGroup } from '../model/Group';
import Ranking from '../model/Ranking';
import ACommonDataProvider from './ACommonDataProvider';
import { IDataProviderOptions, IStatsBuilder } from './ADataProvider';
/**
 * interface what the server side has to provide
 */
export interface IServerData {
    /**
     * sort the dataset by the given description
     * @param ranking
     */
    sort(ranking: Ranking): Promise<number[]>;
    /**
     * returns a slice of the data array identified by a list of indices
     * @param indices
     */
    view(indices: number[]): Promise<any[]>;
    /**
     * returns a sample of the values for a given column
     * @param column
     */
    mappingSample(column: any): Promise<number[]>;
    /**
     * return the matching indices matching the given arguments
     * @param search
     * @param column
     */
    search(search: string | RegExp, column: any): Promise<number[]>;
    stats(indices: number[]): IStatsBuilder;
}
export interface IRemoteDataProviderOptions {
    /**
     * maximal cache size (unused at the moment)
     */
    maxCacheSize: number;
}
/**
 * a remote implementation of the data provider
 */
export default class RemoteDataProvider extends ACommonDataProvider {
    private server;
    private readonly options;
    private readonly cache;
    constructor(server: IServerData, columns?: IColumnDesc[], options?: Partial<IRemoteDataProviderOptions & IDataProviderOptions>);
    getTotalNumberOfRows(): number;
    sortImpl(ranking: Ranking): Promise<IOrderedGroup[]>;
    private loadFromServer;
    view(indices: number[]): Promise<any[]>;
    private computeMissing;
    private loadInCache;
    fetch(orders: number[][]): Promise<IDataRow>[][];
    mappingSample(col: Column): Promise<number[]>;
    searchAndJump(search: string | RegExp, col: Column): void;
    stats(indices: number[]): IStatsBuilder;
}
