import AggregateGroupColumn from '../model/AggregateGroupColumn';
import { AGGREGATE, CANVAS_HEIGHT, cssClass } from '../styles';
var SHIFT = 0;
/** @internal */
var AggregateGroupRenderer = /** @class */ (function () {
    function AggregateGroupRenderer() {
        this.title = 'Default';
    }
    AggregateGroupRenderer.prototype.canRender = function (col) {
        return col instanceof AggregateGroupColumn;
    };
    AggregateGroupRenderer.prototype.create = function (col, context) {
        var width = context.colWidth(col);
        return {
            template: "<div title=\"Collapse Group\"></div>",
            update: function (node, _row, _i, group, meta) {
                node.dataset.meta = meta || undefined;
                node.onclick = function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    col.setAggregated(group, true);
                };
            },
            render: function (ctx, _row, _i, _group, meta) {
                ctx.fillStyle = AGGREGATE.color;
                ctx.fillRect(width - AGGREGATE.width - SHIFT, 0, AGGREGATE.strokeWidth, CANVAS_HEIGHT);
                return Boolean(meta);
            }
        };
    };
    AggregateGroupRenderer.prototype.createGroup = function (col) {
        return {
            template: "<div title=\"Expand Group\"></div>",
            update: function (node, group) {
                node.onclick = function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    col.setAggregated(group, false);
                };
            }
        };
    };
    AggregateGroupRenderer.prototype.createSummary = function (col, context) {
        var cdown = cssClass('icon-caret-down');
        var cright = cssClass('icon-caret-right');
        return {
            template: "<div title=\"(Un)Aggregate All\" class=\"" + cdown + "\"></div>",
            update: function (node) {
                var ranking = col.findMyRanker();
                var right = Boolean(ranking && ranking.getGroups().every(function (g) { return col.isAggregated(g); }));
                node.classList.toggle(cdown, !right);
                node.classList.toggle(cright, right);
                node.onclick = function (evt) {
                    evt.stopPropagation();
                    var ranking = col.findMyRanker();
                    if (!ranking || !context) {
                        return;
                    }
                    var aggregate = node.classList.contains(cdown);
                    node.classList.toggle(cdown, !aggregate);
                    node.classList.toggle(cright, aggregate);
                    context.provider.aggregateAllOf(ranking, aggregate);
                };
            }
        };
    };
    return AggregateGroupRenderer;
}());
export default AggregateGroupRenderer;
//# sourceMappingURL=AggregateGroupRenderer.js.map