import { computeHist, round } from '../internal/math';
import { isCategoricalColumn } from '../model';
import CategoricalColumn from '../model/CategoricalColumn';
import Column from '../model/Column';
import OrdinalColumn from '../model/OrdinalColumn';
import { filterMissingNumberMarkup } from '../ui/missing';
import { interactiveHist } from './CategoricalCellRenderer';
import { ERenderMode } from './interfaces';
import { forEachChild, noRenderer, adaptTextColorToBgColor } from './utils';
import { cssClass, FILTERED_OPACITY } from '../styles';
import { color } from 'd3-color';
/** @internal */
var CategoricalStackedDistributionlCellRenderer = /** @class */ (function () {
    function CategoricalStackedDistributionlCellRenderer() {
        this.title = 'Distribution Bar';
    }
    CategoricalStackedDistributionlCellRenderer.prototype.canRender = function (col, mode) {
        return isCategoricalColumn(col) && mode !== ERenderMode.CELL;
    };
    CategoricalStackedDistributionlCellRenderer.prototype.create = function () {
        return noRenderer;
    };
    CategoricalStackedDistributionlCellRenderer.prototype.createGroup = function (col) {
        var _a = stackedBar(col), template = _a.template, update = _a.update;
        return {
            template: template + "</div>",
            update: function (n, _group, rows) {
                var _a = computeHist(rows, function (r) { return col.getCategory(r); }, col.categories), hist = _a.hist, missing = _a.missing;
                update(n, hist, missing);
            }
        };
    };
    CategoricalStackedDistributionlCellRenderer.prototype.createSummary = function (col, ctx, interactive, unfilteredHist) {
        return (col instanceof CategoricalColumn || col instanceof OrdinalColumn) ? interactiveSummary(col, interactive, unfilteredHist, ctx.idPrefix) : staticSummary(col);
    };
    return CategoricalStackedDistributionlCellRenderer;
}());
export default CategoricalStackedDistributionlCellRenderer;
function staticSummary(col) {
    var _a = stackedBar(col), template = _a.template, update = _a.update;
    return {
        template: template + "</div>",
        update: function (n, hist) {
            n.classList.toggle(cssClass('missing'), !hist);
            if (!hist) {
                return;
            }
            update(n, hist.hist, hist.missing);
        }
    };
}
function interactiveSummary(col, interactive, unfilteredHist, idPrefix) {
    var _a = stackedBar(col, unfilteredHist), template = _a.template, update = _a.update;
    var filterUpdate;
    return {
        template: "" + template + (interactive ? filterMissingNumberMarkup(false, 0, idPrefix) : '') + "</div>",
        update: function (n, hist) {
            if (!filterUpdate) {
                filterUpdate = interactiveHist(col, n);
            }
            var missing = interactive && unfilteredHist ? unfilteredHist.missing : (hist ? hist.missing : 0);
            filterUpdate(missing, col);
            n.classList.toggle(cssClass('missing'), !hist);
            if (!hist) {
                return;
            }
            update(n, hist.hist, hist.missing);
        }
    };
}
function selectedCol(value) {
    var c = color(value);
    c.opacity = FILTERED_OPACITY;
    return c.toString();
}
function stackedBar(col, unfilteredHist) {
    var cats = col.categories.map(function (c) { return ({
        label: c.label,
        color: c.color,
        selected: selectedCol(c.color)
    }); });
    cats.push({ label: 'Missing Values', color: Column.DEFAULT_COLOR, selected: 'transparent' });
    var bins = col.categories.map(function (c) { return "<div class=\"" + cssClass('distribution-bar') + "\" style=\"background-color: " + c.color + "; color: " + adaptTextColorToBgColor(c.color) + "\" title=\"" + c.label + ": 0\" data-cat=\"" + c.name + "\"><span>" + c.label + "</span></div>"; }).join('');
    var updateSingle = function (n, hist, missing) {
        var total = hist.reduce(function (acc, _a) {
            var y = _a.y;
            return acc + y;
        }, missing);
        forEachChild(n, function (d, i) {
            var y = i >= hist.length ? missing : hist[i].y;
            var label = cats[i].label;
            d.style.flexGrow = "" + round(total === 0 ? 0 : y, 2);
            d.title = label + ": " + y;
        });
    };
    var updateUnfiltered = function (n, hist, missing) {
        var uf = unfilteredHist;
        var total = uf.hist.reduce(function (acc, _a) {
            var y = _a.y;
            return acc + y;
        }, uf.missing);
        forEachChild(n, function (d, i) {
            var y = i >= hist.length ? missing : hist[i].y;
            var label = cats[i].label;
            var gY = i >= hist.length ? uf.missing : uf.hist[i].y;
            d.style.flexGrow = "" + round(total === 0 ? 0 : gY, 2);
            d.title = label + ": " + y + " of " + gY;
            var relY = 100 - round(y * 100 / gY, 2);
            d.style.background = relY === 0 ? cats[i].color : (relY === 100 ? cats[i].selected : "linear-gradient(" + cats[i].selected + " " + relY + "%, " + cats[i].color + " " + relY + "%, " + cats[i].color + " 100%)");
        });
    };
    return {
        template: "<div>" + bins + "<div title=\"Missing Values\"></div>",
        update: unfilteredHist ? updateUnfiltered : updateSingle
    };
}
//# sourceMappingURL=CategoricalStackedDistributionlCellRenderer.js.map