import { createHeader, updateHeader } from './header';
import { cssClass, engineCssClass } from '../styles/index';
var RenderColumn = /** @class */ (function () {
    function RenderColumn(c, index, ctx, flags) {
        this.c = c;
        this.index = index;
        this.ctx = ctx;
        this.flags = flags;
        this.renderers = null;
    }
    Object.defineProperty(RenderColumn.prototype, "width", {
        get: function () {
            return this.c.getWidth();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RenderColumn.prototype, "id", {
        get: function () {
            return this.c.id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RenderColumn.prototype, "frozen", {
        get: function () {
            return this.flags.disableFrozenColumns ? false : this.c.frozen;
        },
        enumerable: true,
        configurable: true
    });
    RenderColumn.prototype.singleRenderer = function () {
        if (!this.renderers || !this.renderers.single) {
            return null;
        }
        if (this.renderers.singleTemplate) {
            return this.renderers.singleTemplate.cloneNode(true);
        }
        var elem = this.ctx.asElement(this.renderers.single.template);
        elem.classList.add(cssClass("renderer-" + this.renderers.singleId), cssClass('detail'));
        elem.dataset.renderer = this.renderers.singleId;
        elem.dataset.group = 'd';
        this.renderers.singleTemplate = elem.cloneNode(true);
        return elem;
    };
    RenderColumn.prototype.groupRenderer = function () {
        if (!this.renderers || !this.renderers.group) {
            return null;
        }
        if (this.renderers.groupTemplate) {
            return this.renderers.groupTemplate.cloneNode(true);
        }
        var elem = this.ctx.asElement(this.renderers.group.template);
        elem.classList.add(cssClass("renderer-" + this.renderers.groupId), cssClass('group'));
        elem.dataset.renderer = this.renderers.groupId;
        elem.dataset.group = 'g';
        this.renderers.groupTemplate = elem.cloneNode(true);
        return elem;
    };
    RenderColumn.prototype.summaryRenderer = function () {
        if (!this.renderers || !this.renderers.summary) {
            return null;
        }
        if (this.renderers.summaryTemplate) {
            return this.renderers.summaryTemplate.cloneNode(true);
        }
        var elem = this.ctx.asElement(this.renderers.summary.template);
        elem.classList.add(cssClass('summary'), cssClass('th-summary'), cssClass("renderer-" + this.renderers.summaryId));
        elem.dataset.renderer = this.renderers.summaryId;
        this.renderers.summaryTemplate = elem.cloneNode(true);
        return elem;
    };
    RenderColumn.prototype.createHeader = function () {
        var node = createHeader(this.c, this.ctx, {
            extraPrefix: 'th',
            dragAble: this.flags.advancedUIFeatures,
            mergeDropAble: this.flags.advancedModelFeatures,
            rearrangeAble: this.flags.advancedUIFeatures,
            resizeable: this.flags.advancedUIFeatures
        });
        node.classList.add(cssClass('header'));
        if (!this.flags.disableFrozenColumns) {
            node.classList.toggle(engineCssClass('frozen'), this.frozen);
        }
        if (this.renderers && this.renderers.summary) {
            var summary = this.summaryRenderer();
            node.appendChild(summary);
        }
        this.updateHeader(node);
        return node;
    };
    RenderColumn.prototype.updateHeader = function (node) {
        updateHeader(node, this.c);
        if (!this.renderers || !this.renderers.summary) {
            return;
        }
        var summary = node.querySelector("." + cssClass('summary'));
        var oldRenderer = summary.dataset.renderer;
        var currentRenderer = this.renderers.summaryId;
        if (oldRenderer !== currentRenderer) {
            summary.remove();
            summary = this.summaryRenderer();
            node.appendChild(summary);
        }
        this.renderers.summary.update(summary, this.ctx.statsOf(this.c));
    };
    RenderColumn.prototype.createCell = function (index) {
        var isGroup = this.ctx.isGroup(index);
        var node = isGroup ? this.groupRenderer() : this.singleRenderer();
        this.updateCell(node, index);
        return node;
    };
    RenderColumn.prototype.updateCell = function (node, index) {
        if (!this.flags.disableFrozenColumns) {
            node.classList.toggle(engineCssClass('frozen'), this.frozen);
        }
        var isGroup = this.ctx.isGroup(index);
        // assert that we have the template of the right mode
        // FIXME
        var oldRenderer = node.dataset.renderer;
        var currentRenderer = isGroup ? this.renderers.groupId : this.renderers.singleId;
        var oldGroup = node.dataset.group;
        var currentGroup = (isGroup ? 'g' : 'd');
        if (oldRenderer !== currentRenderer || oldGroup !== currentGroup) {
            node = isGroup ? this.groupRenderer() : this.singleRenderer();
        }
        if (isGroup) {
            var g = this.ctx.getGroup(index);
            this.renderers.group.update(node, g, g.rows);
        }
        else {
            var r = this.ctx.getRow(index);
            this.renderers.single.update(node, r, r.relativeIndex, r.group, r.meta);
        }
        return node;
    };
    RenderColumn.prototype.renderCell = function (ctx, index) {
        var r = this.ctx.getRow(index);
        var s = this.renderers.single;
        return s.render && s.render(ctx, r, r.relativeIndex, r.group, r.meta);
    };
    return RenderColumn;
}());
export default RenderColumn;
//# sourceMappingURL=RenderColumn.js.map