import * as tslib_1 from "tslib";
import AEventDispatcher from '../../internal/AEventDispatcher';
import { cssClass } from '../../styles/index';
import { clear } from '../../internal';
function isItem(v) {
    return v.id !== undefined;
}
var SearchBox = /** @class */ (function (_super) {
    tslib_1.__extends(SearchBox, _super);
    function SearchBox(options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.options = {
            formatItem: function (item) { return item.text; },
            doc: document,
            placeholder: 'Select...',
        };
        _this.values = [];
        Object.assign(_this.options, options);
        _this.node = _this.options.doc.createElement('div');
        _this.node.classList.add(cssClass('search'));
        _this.node.innerHTML = "<input class=\"" + cssClass('search-input') + "\" type=\"search\" placeholder=\"" + _this.options.placeholder + "\">\n    <ul class=\"" + cssClass('search-list') + "\"></ul>";
        _this.search = _this.node.querySelector('input');
        _this.body = _this.node.querySelector('ul');
        _this.search.onfocus = function () { return _this.focus(); };
        _this.search.onblur = function () { return _this.blur(); };
        _this.search.oninput = function () { return _this.filter(); };
        _this.search.onkeydown = function (evt) { return _this.handleKey(evt); };
        return _this;
    }
    Object.defineProperty(SearchBox.prototype, "data", {
        get: function () {
            return this.values;
        },
        set: function (data) {
            this.values = data;
            clear(this.body);
            this.buildDialog(this.body, this.values);
        },
        enumerable: true,
        configurable: true
    });
    SearchBox.prototype.buildDialog = function (node, values) {
        var _this = this;
        values.forEach(function (v) {
            if (isItem(v)) {
                node.insertAdjacentHTML('beforeend', "<li class=\"" + cssClass('search-item') + "\"><span></span></li>");
                var span_1 = node.lastElementChild;
                span_1.onmousedown = function (evt) {
                    // see https://stackoverflow.com/questions/10652852/jquery-fire-click-before-blur-event#10653160
                    evt.preventDefault();
                };
                span_1.onclick = function () { return _this.select(v); };
                span_1.onmouseenter = function () { return _this.highlighted = span_1; };
                span_1.onmouseleave = function () { return _this.highlighted = null; };
            }
            else {
                node.insertAdjacentHTML('beforeend', "<li class=\"" + cssClass('search-group') + "\"><span></span><ul></ul></li>");
                var ul = node.lastElementChild.lastElementChild;
                _this.buildDialog(ul, v.children);
            }
            var item = node.lastElementChild.firstElementChild;
            item.innerHTML = _this.options.formatItem(v, item);
        });
    };
    SearchBox.prototype.handleKey = function (evt) {
        var KEYS = {
            ESC: 27,
            ENTER: 13,
            UP: 38,
            DOWN: 40
        };
        switch (evt.which) {
            case KEYS.ESC:
                this.search.blur();
                break;
            case KEYS.ENTER:
                var h = this.highlighted;
                if (h) {
                    h.click();
                }
                break;
            case KEYS.UP:
                this.highlightPrevious();
                break;
            case KEYS.DOWN:
                this.highlightNext();
                break;
        }
    };
    SearchBox.prototype.select = function (item) {
        this.search.value = ''; // reset
        this.search.blur();
        this.fire(SearchBox.EVENT_SELECT, item);
    };
    SearchBox.prototype.focus = function () {
        this.body.style.width = this.search.offsetWidth + "px";
        this.highlighted = this.body.firstElementChild || null;
        this.node.classList.add(cssClass('search-open'));
    };
    Object.defineProperty(SearchBox.prototype, "highlighted", {
        get: function () {
            return this.body.querySelector("." + cssClass('search-highlighted')) || null;
        },
        set: function (value) {
            var old = this.highlighted;
            if (old === value) {
                return;
            }
            if (old) {
                old.classList.remove(cssClass('search-highlighted'));
            }
            if (value) {
                value.classList.add(cssClass('search-highlighted'));
            }
        },
        enumerable: true,
        configurable: true
    });
    SearchBox.prototype.highlightNext = function () {
        var h = this.highlighted;
        if (!h || h.classList.contains(cssClass('hidden'))) {
            this.highlighted = this.body.querySelector("." + cssClass('search-item') + ":not(." + cssClass('hidden') + ")") || null;
            return;
        }
        var items = Array.from(this.body.querySelectorAll("." + cssClass('search-item') + ":not(." + cssClass('hidden') + ")"));
        var index = items.indexOf(h);
        this.highlighted = items[index + 1] || null;
    };
    SearchBox.prototype.highlightPrevious = function () {
        var h = this.highlighted;
        var items = Array.from(this.body.querySelectorAll("." + cssClass('search-item') + ":not(." + cssClass('hidden') + ")"));
        if (!h || h.classList.contains(cssClass('hidden'))) {
            this.highlighted = items[items.length - 1] || null;
            return;
        }
        var index = items.indexOf(h);
        this.highlighted = items[index - 1] || null;
    };
    SearchBox.prototype.blur = function () {
        this.search.value = '';
        // clear filter
        this.filterResults(this.body, '');
        this.node.classList.remove(cssClass('search-open'));
    };
    SearchBox.prototype.filter = function () {
        var empty = this.filterResults(this.body, this.search.value.toLowerCase());
        this.body.classList.toggle(cssClass('search-empty'), empty);
    };
    SearchBox.prototype.filterResults = function (node, text) {
        var _this = this;
        if (text === '') {
            // show all
            Array.from(node.querySelectorAll("." + cssClass('hidden'))).forEach(function (d) { return d.classList.remove(cssClass('hidden')); });
            return false;
        }
        var children = Array.from(node.children);
        children.forEach(function (d) {
            var content = d.firstElementChild.innerHTML.toLowerCase();
            var hidden = !content.includes(text);
            if (d.classList.contains(cssClass('search-group'))) {
                var ul = d.lastElementChild;
                var allChildrenHidden = _this.filterResults(ul, text);
                hidden = hidden && allChildrenHidden;
            }
            d.classList.toggle(cssClass('hidden'), hidden);
        });
        return children.every(function (d) { return d.classList.contains(cssClass('hidden')); });
    };
    SearchBox.prototype.createEventList = function () {
        return _super.prototype.createEventList.call(this).concat([SearchBox.EVENT_SELECT]);
    };
    SearchBox.prototype.on = function (type, listener) {
        return _super.prototype.on.call(this, type, listener);
    };
    SearchBox.EVENT_SELECT = 'select';
    return SearchBox;
}(AEventDispatcher));
export default SearchBox;
//# sourceMappingURL=SearchBox.js.map